#pragma once

#include <glad/glad.h>

#include <optional>
#include <vector>

#include "vec3f.h"

namespace openGL {
namespace rendering {

class AxisBuffer {
public:
  ~AxisBuffer();

  AxisBuffer(AxisBuffer const &) = delete;
  AxisBuffer &operator=(AxisBuffer const &) = delete;

  AxisBuffer(AxisBuffer &&other);
  AxisBuffer &operator=(AxisBuffer &&);

  void bindVAO();
  void setupVAO();
  void setupAxisBuffer(std::vector<math::Vec3f> const &vertices);

  size_t size() const;

private:
  AxisBuffer(GLuint vaoID, GLuint vertexColorBufferID);

  void release();
  void bindBuffer();

  friend std::optional<AxisBuffer> makeAxisBuffer();

private:
  GLuint m_vaoID = 0;
  GLuint m_vertexColorBufferID = 0;

  GLuint m_bufferSize = 0;
};

std::optional<AxisBuffer> makeAxisBuffer();

} // namespace rendering
} // namespace openGL
