#pragma once

#include <glad/glad.h>

#include <optional>
#include <vector>

#include "vec3f.h"

namespace openGL {
namespace rendering {

class CurveBuffer {
public:
  ~CurveBuffer();

  CurveBuffer(CurveBuffer const &) = delete;
  CurveBuffer &operator=(CurveBuffer const &) = delete;

  CurveBuffer(CurveBuffer &&other);
  CurveBuffer &operator=(CurveBuffer &&);

  void bindVAO();
  void setupVAO();
  void setupCurveBuffers(std::vector<math::Vec3f> const &vertices,
                         std::vector<math::Vec3f> const &colors);
  void setupCurveBuffers(std::vector<math::Vec3f> const &vertices,
                         math::Vec3f const &defaultColorRGB);
  void loadColorBuffer(std::vector<math::Vec3f> const &colors);

  size_t size() const;

private:
  CurveBuffer(GLuint vaoID, GLuint vertexBufferID, GLuint colorBufferID);

  void release();
  void bindVertex();
  void bindColor();

  friend std::optional<CurveBuffer> makeCurveBuffer();

private:
  GLuint m_vaoID = 0;
  GLuint m_vertexBufferID = 0;
  GLuint m_colorBufferID = 0;

  GLuint m_bufferSize = 0;
};

std::optional<CurveBuffer> makeCurveBuffer();

} // namespace rendering
} // namespace openGL
