#pragma once

namespace openGL {
namespace glslSource {

constexpr char *PointVS_Source = R"GLSL(
  #version 330 core
  layout(location = 0) in vec3 vertexPosition_modelspace;
  layout(location = 1) in vec3 inVertexColor;

  uniform mat4 MVP;
  uniform float pointSize;

  out vec3 vertexColor;

  void main() {
    gl_Position = MVP * vec4(vertexPosition_modelspace, 1.f);
    gl_PointSize = pointSize;
    vertexColor = inVertexColor;
  }
  )GLSL";

constexpr char *PointFS_Source = R"GLSL(
  #version 330 core
  in vec3 vertexColor;

  out vec4 color;

  void main() {
    vec2 dir = gl_PointCoord - vec2(0.5,0.5);
    float r2 = dot(dir,dir);
    if( r2 > 0.25 )
      discard;
    color = vec4( mix(vertexColor, vec3(0,0,0), r2/0.25), 1.f);
  }
  )GLSL";

constexpr char *LineVS_Source = R"GLSL(
  #version 330 core
  layout( location = 0 ) in vec3 vert_modelSpace;
  layout( location = 1 ) in vec3 inVertexColor;
  uniform mat4 MVP;
  out vec3 vertexColor;
  void main()
  {
    gl_Position = MVP * vec4( vert_modelSpace, 1.f );
    vertexColor = inVertexColor;
  }
)GLSL";

constexpr char *LineGS_Source = R"GLSL(
  #version 330 core
  layout( lines ) in;
  layout( triangle_strip, max_vertices = 4 ) out;
  in vec3 vertexColor[];

  out VertexData
  {
    vec3 color;
    vec2 textureCoord;
  } outVertex;
  uniform vec2 windowSize;
  uniform float lineThickness;
  void main()
  {
    vec4 posA = gl_in[0].gl_Position;
    vec4 posB = gl_in[1].gl_Position;

    vec2 screenA = vec2(posA.xy/posA.w) * windowSize;
    vec2 screenB = vec2(posB.xy/posB.w) * windowSize;

    float depthA = posA.z/posA.w;
    float depthB = posB.z/posB.w;

    vec2 norm = screenA - screenB;
    norm = normalize(vec2(-norm.y, norm.x)) * lineThickness;

    //vertex 0
    gl_Position	= vec4((screenA - norm) / windowSize, depthA, 1.);
    outVertex.textureCoord = vec2(0.,1.);
    outVertex.color = vertexColor[0];
    EmitVertex();

    //vertex 1
    gl_Position	= vec4((screenA + norm) / windowSize, depthA, 1.);
    outVertex.textureCoord = vec2(0.,0.);
    outVertex.color = vertexColor[0];
    EmitVertex();

    //vertex 2
    gl_Position	= vec4((screenB - norm) / windowSize, depthB, 1.);
    outVertex.textureCoord = vec2(1.,1.);
    outVertex.color = vertexColor[1];
    EmitVertex();

    //vertex 4
    gl_Position	= vec4((screenB + norm) / windowSize, depthB, 1.);
    outVertex.textureCoord = vec2(1.,0.);
    outVertex.color = vertexColor[1];
    EmitVertex();

    EndPrimitive();
  }
 )GLSL";

constexpr const char *LineFS_Source = R"GLSL(
  #version 330 core
  in VertexData
  {
    vec3 color;
    vec2 textureCoord;
  } inVertex;

  out vec4 color;

  void main()
  {
    float dir = inVertex.textureCoord.y - 0.5;
    float r2 = dir * dir;
    color = vec4( mix(inVertex.color, vec3(0,0,0), r2/0.25), 1.f);
  }
)GLSL";

constexpr char *AxisVS_Source = R"GLSL(
  #version 330 core
  layout( location = 0 ) in vec3 vert_modelSpace;
  layout( location = 1 ) in vec3 inVertexColor;
  uniform mat4 MVP;
  out vec3 vertexColor;
  void main()
  {
    gl_Position = MVP * vec4( vert_modelSpace, 1.f );
    vertexColor = inVertexColor;
  }
)GLSL";

constexpr const char *AxisFS_Source = R"GLSL(
  #version 330 core
  in vec3 vertexColor;
  out vec4 color;

  void main()
  {
    color = vec4(vertexColor, 1.f);
  }
)GLSL";

} // namespace glslSource
} // namespace openGL
