#pragma once

#include <iosfwd>

#include "mat4f.h"
#include "vec3f.h"

namespace math {

// struct declaration
struct Vec4f {
  union {
    struct {
      float m_x;
      float m_y;
      float m_z;
      float m_w;
    };
    // quite often we us this as Homogenous coordinates of a point (w = 1)
    float m_coord[4] = {0.f, 0.f, 0.f, 1.f};
  };

  Vec4f() = default;
  Vec4f(float x, float y, float z, float w);
  Vec4f(Vec3f const &v);
  Vec4f(Vec3f const &v, float w);

  operator Vec3f() const;

  /*
   * Mutating member functions
   */
  Vec4f &operator+=(Vec4f const &rhs);
  Vec4f &operator-=(Vec4f const &rhs);
  Vec4f &operator*=(float rhs);
  Vec4f &operator/=(float rhs);

  float &operator[](size_t id);
  float const &operator[](size_t id) const;

  float *data();
  float const *data() const;

  Vec4f &normalize();

  void zero();
};

// Free function declarations
Vec4f operator+(Vec4f const &a, Vec4f const &b);
Vec4f operator-(Vec4f const &a, Vec4f const &b);
Vec4f operator*(float s, Vec4f v);
Vec4f operator*(Vec4f v, float s);
Vec4f operator/(Vec4f v, float s);
Vec4f operator-(Vec4f v);

float operator*(Vec4f const &a, Vec4f const &b);
float dot(Vec4f const &a, Vec4f const &b);

float norm(Vec4f const &v);
float normSquared(Vec4f const &v);
Vec4f normalized(Vec4f v);

Vec4f operator*(Mat4f const &lhs, Vec4f const &rhs);

std::istream &operator>>(std::istream &in, Vec4f &v);
std::ostream &operator<<(std::ostream &out, Vec4f const &v);

} // namespace math
