#pragma once

#include "mat4f.h"
#include "vec2i.h"
#include "vec3f.h"

namespace openGL {
namespace rendering {

class Viewport {
public:
  Viewport(math::Vec2i size);
  Viewport(math::Vec2i size, float fovDegrees, float nearZ, float farZ);

  void windowSize(math::Vec2i size);
  void framebufferSize(math::Vec2i size);
  void nearPlaneDistance(float distance);
  void farPlaneDistance(float distance);
  void fieldOfViewDegrees(float degrees);

  math::Vec2i windowSize() const;
  math::Vec2i framebufferSize() const;
  int width() const;
  int height() const;
  float aspectRatio() const;
  float nearPlaneDistance() const;
  float farPlaneDistance() const;
  float fieldOfViewDegrees() const;

private:
  math::Vec2i m_windowSize;
  math::Vec2i m_framebufferSize;

  float m_fovDegrees = 60.f;
  float m_nearZ = 0.01f;
  float m_farZ = 100.f;
};

math::Mat4f makeInverseProjectionMatrix(Viewport const &viewport);

math::Mat4f makeProjectionMatrix(Viewport const &viewport);

math::Vec3f normalizedDeviceCoordinates(math::Vec2i const &pixelPositions,
                                        Viewport const &Viewport);

math::Vec2i pixelPosition(math::Vec3f const &normalizedDeviceCoordinates,
                          Viewport const &Viewport);

math::Vec3f worldDimensions(Viewport const &viewport);

} // namespace rendering
} // namespace openGL
