#include "opengltools.h"

namespace openGL {

math::Vec3f unprojectToWorld(math::Vec2i const &pixelPosition,
                             scene::Camera const &camera,
                             rendering::Viewport const &viewport) {
  using namespace math;

  Vec3f posNDC =
      rendering::normalizedDeviceCoordinates(pixelPosition, viewport);

  Vec3f worldDimensions = rendering::worldDimensions(viewport);

  Vec3f offset = componentMultiplication(posNDC, worldDimensions);

  return camera.localPos() +
         camera.localForward() * viewport.nearPlaneDistance() +
         camera.localUp() * offset.m_y + camera.localRight() * offset.m_x;
}

math::Vec4f perspectiveDivide(math::Vec4f v) { return v /= v.m_w; }

/**
 * For derivation and images take a look at
 * http://antongerdelan.net/opengl/raycasting.html
 */

math::geometry::Ray rayFromPixelPosition(math::Vec2i const &pixelPosition,
                                         scene::Camera const &camera,
                                         rendering::Viewport const &viewport) {
  using namespace math;

  Vec3f normalizedDeviceCoordinates =
      rendering::normalizedDeviceCoordinates(pixelPosition, viewport);

  Vec4f rayClip(normalizedDeviceCoordinates.m_x,
                normalizedDeviceCoordinates.m_y, -1.f, 1.f);

  Vec4f rayEye = rendering::makeInverseProjectionMatrix(viewport) * rayClip;
  rayEye.m_z = -1.f;
  rayEye.m_w = 0.f;

  Vec3f rayWorld = scene::makeInverseViewMatrix(camera) * rayEye;
  rayWorld.normalize();

  return {camera.localPos(), rayWorld};
}

} // namespace openGL
