#include "plane.h"

namespace math {
namespace geometry {

Plane::Plane(Vec3f origin) : m_origin(origin) {}

Plane::Plane(Vec3f origin, Vec3f normal) : m_origin(origin), m_normal(normal) {}

// useful to use optional here as there is no known sentinel value w/o imposing
// restrictions on ray-plane intersection (i.e. maybe we care about t <= 0,
// maybe not)

std::optional<Ray::Intersection>
intersectRayPlane(math::Vec3f const &pO, math::Vec3f const &pN,
                  math::Vec3f const &rO, math::Vec3f const &rD, float epsilon) {

  float denom = rD * pN;
  if (std::abs(denom) < epsilon)
    return std::nullopt; // vectors are parallel == no intersection

  float t = (pN * (pO - rO)) / denom;

  return Ray::Intersection{t, rO + rD * t};
}

std::optional<Ray::Intersection> intersect(Plane const &plane, Ray const &ray,
                                           float epsilon) {
  return intersectRayPlane(plane.m_origin, plane.m_normal, ray.m_origin,
                           ray.m_direction, epsilon);
}

} // namespace geometry
} // namespace math
