#include "resourcemanager.h"

namespace openGL {
namespace rendering {

ResourceManager::ResourceManager() {}

// PROGRAM MANAGER
openGL::rendering::Program &ResourceManager::getProgram(int idx) {
  return m_program[idx];
}

int ResourceManager::pushProgram(openGL::rendering::Program &&program) {
  int idx = m_program.size();
  m_program.push_back(std::move(program));
  return idx;
}

size_t ResourceManager::programCount() const { return m_program.size(); }

// CURVE BUFFER MANAGER
openGL::rendering::CurveBuffer &ResourceManager::getCurveBuffer(int idx) {
  return m_curveBuffer[idx];
}

int ResourceManager::pushCurveBuffer(
    openGL::rendering::CurveBuffer &&curveBuffer) {
  int idx = m_curveBuffer.size();
  m_curveBuffer.push_back(std::move(curveBuffer));
  return idx;
}

size_t ResourceManager::curveBufferCount() const {
  return m_curveBuffer.size();
}

// AXIS BUFFER MANAGER
openGL::rendering::AxisBuffer &ResourceManager::getAxisBuffer(int idx) {
  return m_axisBuffer[idx];
}

int ResourceManager::pushAxisBuffer(
    openGL::rendering::AxisBuffer &&axisBuffer) {
  int idx = m_axisBuffer.size();
  m_axisBuffer.push_back(std::move(axisBuffer));
  return idx;
}

size_t ResourceManager::axisBufferCount() const { return m_axisBuffer.size(); }

void ResourceManager::clear() {
  m_program.clear();
  m_curveBuffer.clear();
  m_axisBuffer.clear();
}

} // namespace rendering
} // namespace openGL
