#include "vec2i.h"

#include <cmath>
#include <iostream>

namespace math {

Vec2i::Vec2i(int x, int y) : m_x(x), m_y(y) {}

int &Vec2i::operator[](int idx) { return m_coord[idx]; }
int const &Vec2i::operator[](int idx) const { return m_coord[idx]; }

Vec2i &Vec2i::operator+=(Vec2i const &other) {
  m_x += other.m_x;
  m_y += other.m_y;
  return *this;
}
Vec2i &Vec2i::operator-=(Vec2i const &other) {
  m_x -= other.m_x;
  m_y -= other.m_y;
  return *this;
}

Vec2i &Vec2i::operator*=(int factor) {
  m_x *= factor;
  m_y *= factor;
  return *this;
}

int *Vec2i::data() { return &m_coord[0]; }
int const *Vec2i::data() const { return &m_coord[0]; }

// Free functions
Vec2i operator+(Vec2i const &a, Vec2i const &b) {
  return {a.m_x + b.m_x, a.m_y + b.m_y};
}

Vec2i operator-(Vec2i const &a, Vec2i const &b) {
  return {a.m_x - b.m_x, a.m_y - b.m_y};
}

Vec2i operator*(Vec2i const &v, int s) { return {s * v.m_x, s * v.m_y}; }

Vec2i operator*(int s, Vec2i const &v) { return {s * v.m_x, s * v.m_y}; }

Vec2i operator-(Vec2i v) {
  v.m_x = -v.m_x;
  v.m_y = -v.m_y;
  return v;
}
float norm(Vec2i const &v) { return std::sqrt(v.m_x * v.m_x + v.m_y * v.m_y); }

float normSquared(Vec2i const &v) { return v.m_x * v.m_x + v.m_y * v.m_y; }

float distance(Vec2i const &a, Vec2i const &b) { return norm(a - b); }

float distanceSquared(Vec2i const &a, Vec2i const &b) {
  return normSquared(a - b);
}

float operator*(Vec2i const &a, Vec2i const &b) {
  return a.m_x * b.m_x + a.m_y * b.m_y;
}

float dotProduct(Vec2i const &a, Vec2i const &b) {
  return a.m_x * b.m_x + a.m_y * b.m_y;
}

bool operator<(Vec2i const &a, Vec2i const &b) {
  if (a.m_x == b.m_x) {
    if (a.m_y == b.m_y) {
      return false;
    } else {
      return a.m_y < b.m_y;
    }
  } else {
    return a.m_x < b.m_x;
  }
}

std::ostream &operator<<(std::ostream &out, Vec2i const &v) {
  return out << v.m_x << " " << v.m_y;
}

std::istream &operator>>(std::istream &in, Vec2i &v) {
  return in >> v.m_x >> v.m_y;
}

} // namespace math
